<?php
namespace SF\Sfcategory\Hook;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */
use TYPO3\CMS\Core\DataHandling\DataHandler;
use TYPO3\CMS\Core\Resource\ResourceFactory;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 */
class DataHandlerHook
{
    /**
     * @var string
     */
    protected $pattern = '@^(?P<prefix>PI|PR|PP)_.*?_?(?P<country>[a-zA-Z]{2,2})?\.[a-zA-Z]{2,4}$@i';
    
    /**
     * @var array
     */
    protected $lang2iso = array(
        '0' => 'de',
        '1' => 'en',
        '2' => 'us',
        '3' => 'fr',
        '4' => 'es',
        '5' => 'pt',
        '6' => 'ru',
        '7' => 'zh',
    );
    
    /**
     * @param array $incomingFieldArray
     * @param string $table
     * @param string $id
     * @param DataHandler $dataHandler
     *
     * @return void
     */
    public function processDatamap_preProcessFieldArray(array &$incomingFieldArray, $table, $id, DataHandler $dataHandler)
    {
        if (
            $table === 'sys_file_reference' &&
            array_key_exists('tt_content', $dataHandler->datamap) &&
            empty($incomingFieldArray['title'])
        ) {
            $sysFileUid = (int)str_replace('sys_file_', '', $incomingFieldArray['uid_local']);
            try {
                $resourceFactory = ResourceFactory::getInstance();
                $file = $resourceFactory->getFileObject($sysFileUid);
                $title = $this->getTitle($file->getName(), (int)$incomingFieldArray['sys_language_uid']);
                if (!empty($title)) {
                    $incomingFieldArray['title'] = $title;
                }
            } catch (\Exception $e) {
                // file not found, permission problems, what ever
            }
        }
    }
    
    /**
     * Get new title for file
     *
     * @param string $filename
     * @param int $language
     *
     * @return string
     */
    protected function getTitle($filename, $language = 0)
    {
        $title = '';
        $matches = array();
        $iso = $this->lang2iso[$language];
        $languageFactory = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Localization\LocalizationFactory::class);
        $LOCAL_LANG = $languageFactory->getParsedData('fileadmin/Language/locallang.xlf', $iso);
        if (!preg_match($this->pattern, $filename, $matches)) {
            return '';
        }
        if (isset($matches['prefix'])) {
            switch (strtoupper($matches['prefix'])) {
                case 'PI':
                    // PI files have a 2 char country code
                    if (isset($matches['country'])) {
                        $title = sprintf(
                            '%s - %s',
                            strtoupper($matches['country']),
                            $LOCAL_LANG[$iso]['chooseLanguage'][0]['target']
                        );
                    }
                    break;
                case 'PR':
                    $title = 'Veroeffentlichung';
                    break;
                case 'PP':
                    $title = 'Bilder';
                    break;
                default:
                    // do nothing
            }
        }
        return $title;
    }
}
