<?php
namespace SF\Sfcategory\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2013 Stefan Froemken <froemken@gmail.com>
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/
use SF\Sfcategory\Domain\Model\Car;
use SF\Sfcategory\Domain\Model\Person;
use TYPO3\CMS\Core\Messaging\AbstractMessage;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;

/**
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 */
class CarController extends ActionController {

    /**
     * carRepository
     *
     * @var \SF\Sfcategory\Domain\Repository\CarRepository
     *
     * @inject
     */
    protected $carRepository;

    /**
     * personRepository
     *
     * @var \SF\Sfcategory\Domain\Repository\PersonRepository
     *
     * @inject
     */
    protected $personRepository;

    /**
     * action list
     *
     * @return void
     */
    public function listAction() {
        $cars = $this->carRepository->findAll();
        $this->view->assign('cars', $cars);
    }

    /**
     * action show
     *
     * @param \SF\Sfcategory\Domain\Model\Car $car
     *
     * @return void
     */
    public function showAction(\SF\Sfcategory\Domain\Model\Car $car) {
        $this->view->assign('car', $car);
    }

    /**
     * action new
     *
     * @return void
     */
    public function newAction() {
        $this->view->assign('car', $this->objectManager->get('SF\\Sfcategory\\Domain\\Model\\Car'));
    }

    /**
     * Create english car
     *
     * @return void
     */
    public function createEnglishAction() {
        $this->clearCacheOnError();
        $cars = $this->carRepository->countAll();
        if ($cars) {
            $this->addFlashMessage('An english car was already created.', 'Error', AbstractMessage::ERROR);
        } else {
            $englishCar = new Car();
            $englishCar->setTitle('Car');
            // _languageUid will be 0
            // _localizedUid will be 0
            $this->carRepository->add($englishCar);
            $this->addFlashMessage('English car was created.', 'Creation successfully', AbstractMessage::OK);
        }
        $this->redirect('list');
    }

    /**
     * Create german car
     *
     * @return void
     */
    public function createGermanAction() {
        $this->clearCacheOnError();
        /** @var Car $car */
        $car = $this->carRepository->findAll()->getFirst();
        if ($car) {
            $germanCar = new Car();
            $germanCar->setTitle('Auto');
            // FIXME _setProperty is marked for internal use only and not meant to be called in userland code
            $germanCar->_setProperty('_languageUid', 1);
            $germanCar->setL10nParent($car->getUid());
            $this->carRepository->add($germanCar);
            $this->addFlashMessage('German car was created.', 'Creation successfully', AbstractMessage::OK);
        } else {
            $this->addFlashMessage('Please create an english car first.', 'Error', AbstractMessage::ERROR);
        }
        $this->redirect('list');
    }

    /**
     * action edit
     *
     * @param \SF\Sfcategory\Domain\Model\Car $car
     *
     * @return void
     */
    public function editAction(\SF\Sfcategory\Domain\Model\Car $car) {
        $this->view->assign('car', $car);
    }

    /**
     * Add new Person to car
     *
     * @param Car $car
     *
     * @return void
     */
    public function addNewPersonAction(Car $car)
    {
        $this->clearCacheOnError();
        if ($car->getPersons()->count()) {
            $this->addFlashMessage('A person was already added to car', 'Error', AbstractMessage::ERROR);
        } else {
            if (empty($car->_getProperty('_localizedUid'))) {
                // default language
                $person = new Person();
                $person->setName('Stefano');
                $car->addPerson($person);
                $this->carRepository->update($car);
                $this->addFlashMessage('A person was added to the car.', 'Update successfully');
            }
        }

        $this->redirect('show', null, null, ['car' => $car]);
    }

    /**
     * Add translated Person to car
     *
     * @param Car $car
     *
     * @return void
     */
    public function addTranslatedPersonAction(Car $car)
    {
        $this->clearCacheOnError();
        if ($car->getPersons()->count()) {
            // translation
            $car->getPersons()->rewind();
            /** @var Person $englishPerson */
            $englishPerson = $car->getPersons()->current();
            $person = new Person();
            $person->setName('Stefan');
            $person->_setProperty('_localizedUid', $englishPerson->getUid());
            $person->_setProperty('_languageUid', 1);

            $this->personRepository->add($person);

            $this->addFlashMessage('A translated person was added to the car.', 'Update successfully');
        }

        $this->redirect('show', null, null, ['car' => $car]);
    }

    /**
     * action update
     *
     * @param \SF\Sfcategory\Domain\Model\Car $car
     *
     * @return void
     */
    public function updateAction(\SF\Sfcategory\Domain\Model\Car $car) {
        $this->carRepository->update($car);
        $this->addFlashMessage('Car was updated.', 'Update successfully');
        $this->redirect('list');
    }

    /**
     * action delete
     *
     * @param \SF\Sfcategory\Domain\Model\Car $car
     *
     * @return void
     */
    public function deleteAction(\SF\Sfcategory\Domain\Model\Car $car) {
        $this->carRepository->remove($car);
        $this->addFlashMessage('Car was removed.', 'Deletion successfully');
        $this->redirect('list');
    }
}
